<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="utf-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <!-- Latest compiled and minified CSS -->
    <link rel="stylesheet" href="https://maxcdn.bootstrapcdn.com/bootstrap/3.3.6/css/bootstrap.min.css" integrity="sha384-1q8mTJOASx8j1Au+a5WDVnPi2lkFfwwEAa8hDDdjZlpLegxhjVME1fgjWPGmkzs7" crossorigin="anonymous">
    <title>Bot Status</title>
</head>
<body>
<div class="container">
    <div class="row">
        <div class="col-sm-*">
            <h2 class="text-center">@{{ bot_name }} Status</h2>
        </div>
    </div>
    <div class="row">
        <div class="col-sm-3">
            Running version {{ version }}-{{ commit }}
        </div>
    </div>
    <div class="row">
        <div class="col-sm-3">
            Uptime: {% if uptime.days > 0 %}{{ uptime.days|pluralize('day') }} {% endif %}{% if uptime.hours > 0 %}{{ uptime.hours|pluralize('hour') }} {% endif %}{% if uptime.minutes > 0 %}{{ uptime.minutes|pluralize('minute') }} {% endif %}{{ uptime.seconds|pluralize('second') }}
        </div>
    </div>
    <div class="row">
        <div class="col-sm-*">
            <h3>Running Plugins</h3>
        </div>
    </div>
    <div class="row">
        <div class="col-sm-3"><b>Plugin Name</b></div>
        <div class="col-sm-3"><b>Plugin Version</b></div>
    </div>
    {% for plugin in plugins %}
    <div class="row">
        <div class="col-sm-3">{{ plugin.name }}</div>
        <div class="col-sm-3">{{ plugin.version }}-{{ plugin.commit }}</div>
    </div>
    {% endfor %}
</div>
</body>
</html>